@@ -3,8 +3,12 @@
 from dataclasses import dataclass
 
 from docutils import nodes
+from sphinx.builders import Builder
+
+from sphinx_codeautolink.warn import logger, warn_type
 
 from .directive import DeferredExamples
+from .translation import tr
 
 
 @dataclass
@@ -56,10 +60,19 @@ class CodeRefsVisitor(nodes.SparseNodeVisitor):
     """Replace :class:`DeferredCodeReferences` with table of concrete references."""
 
     def __init__(
-        self, *args, code_refs: dict[str, list[CodeExample]], **kwargs
+        self,
+        *args,
+        code_refs: dict[str, list[CodeExample]],
+        docname: str,
+        builder: Builder,
+        warn_no_backreference: bool = False,
+        **kwargs,
     ) -> None:
         super().__init__(*args, **kwargs)
         self.code_refs = code_refs
+        self.docname = docname
+        self.builder = builder
+        self.warn_no_backreference = warn_no_backreference
 
     def unknown_departure(self, node) -> None:
         """Ignore unknown nodes."""
@@ -71,7 +84,7 @@ class CodeRefsVisitor(nodes.SparseNodeVisitor):
 
         items = []
         for ref in self.code_refs.get(node.ref, []):
-            link = ref.document + ".html"
+            link = self.builder.get_relative_uri(self.docname, ref.document)
             if ref.ref_id is not None:
                 link += f"#{ref.ref_id}"
             items.append((link, " / ".join(ref.headings)))
@@ -79,6 +92,11 @@ class CodeRefsVisitor(nodes.SparseNodeVisitor):
         items = sorted(set(items))
 
         if not items:
+            if self.warn_no_backreference:
+                msg = f"No backreference for: '{node.ref}'"
+                logger.warning(
+                    msg, type=warn_type, subtype="no_backreference", location=node
+                )
             # Remove surrounding paragraph too
             node.parent.parent.remove(node.parent)
             return
@@ -100,7 +118,7 @@ class CodeRefsVisitor(nodes.SparseNodeVisitor):
             entry = nodes.entry()
             row += entry
             title = nodes.paragraph()
-            title += nodes.Text("References to ")
+            title += nodes.Text(tr("References to") + " ")
             title += orig_ref
             entry += title
 
@@ -120,7 +138,7 @@ class CodeRefsVisitor(nodes.SparseNodeVisitor):
         if node.collapse:
             details = DetailsNode()
             summary = SummaryNode()
-            summary += nodes.Text("Expand for references to ")
+            summary += nodes.Text(tr("Expand for references to") + " ")
             summary += orig_ref
             details += summary
             details += table
