@@ -9,7 +9,7 @@ import pytest
 from bs4 import BeautifulSoup
 from sphinx.cmd.build import main as sphinx_main
 
-from ._check import check_link_targets
+from ._check import check_link_targets, check_reference_targets_exist
 
 # Insert test package root to path for all tests
 sys.path.insert(0, str(Path(__file__).parent / "src"))
@@ -26,6 +26,7 @@ autodoc_default_options = {
 }
 codeautolink_warn_on_missing_inventory = True
 codeautolink_warn_on_failed_resolve = True
+codeautolink_warn_on_no_backreference = False
 """
 
 any_whitespace = re.compile(r"\s*")
@@ -191,6 +192,7 @@ But edited.
 
 
 def test_build_twice_and_delete_one_file(tmp_path: Path):
+    conf = default_conf + "\nsuppress_warnings = ['toc.not_readable']"
     index = """
 Test project
 ------------
@@ -212,7 +214,7 @@ Another
 .. autolink-examples:: test_project.bar
 """
 
-    files = {"conf.py": default_conf, "index.rst": index, "another.rst": another}
+    files = {"conf.py": conf, "index.rst": index, "another.rst": another}
     _sphinx_build(tmp_path, "html", files)
     (tmp_path / "src" / "another.rst").unlink()
     _sphinx_build(tmp_path, "html", {})
@@ -239,7 +241,7 @@ Test project
     def raise_nomsg(*_, **__):
         raise ValueError
 
-    target = "sphinx_codeautolink.extension.CodeBlockAnalyser"
+    target = "sphinx_codeautolink.parse.ImportTrackerVisitor"
     with pytest.raises(RuntimeError), patch(target, raise_msg):
         _sphinx_build(tmp_path, "html", files)
 
@@ -276,9 +278,147 @@ Test project
 
     for file in subfiles:
         assert_links(result_dir / (file + ".html"), links)
+
     assert check_link_targets(result_dir) == n_subfiles * len(links)
 
 
+def test_skip_identical_code(tmp_path: Path):
+    """Code skipped and then linked in an identical block after."""
+    index = """
+Test project
+============
+
+.. autolink-skip::
+.. code:: python
+
+   import test_project
+   test_project.bar()
+
+.. code:: python
+
+   import test_project
+   test_project.bar()
+
+.. automodule:: test_project
+"""
+    files = {"conf.py": default_conf, "index.rst": index}
+    result_dir = _sphinx_build(tmp_path, "html", files)
+
+    index_html = result_dir / "index.html"
+    text = index_html.read_text("utf-8")
+    soup = BeautifulSoup(text, "html.parser")
+    blocks = list(soup.select(".highlight-python"))
+    assert len(blocks) == 2
+    assert "sphinx-codeautolink-a" not in str(blocks[0])
+    assert "sphinx-codeautolink-a" in str(blocks[1])
+
+
+def test_dirhtml_builder(tmp_path: Path):
+    index = """
+Test project
+============
+
+.. toctree::
+   :maxdepth: 2
+
+   page1/index
+   page2
+   subdir/page3
+
+Index Page Code
+---------------
+
+.. code:: python
+
+   import test_project
+   test_project.bar()
+
+.. automodule:: test_project
+"""
+
+    page = """
+Page {idx}
+===========
+
+.. code:: python
+
+   import test_project
+   test_project.bar()
+
+.. autolink-examples:: test_project.bar
+"""
+
+    files = {
+        "conf.py": default_conf,
+        "index.rst": index,
+        "page1/index.rst": page.format(idx=1),
+        "page2.rst": page.format(idx=2),
+        "subdir/page3.rst": page.format(idx=3),
+    }
+    links = ["test_project", "test_project.bar"]
+
+    result_dir = _sphinx_build(tmp_path, "dirhtml", files)
+
+    assert_links(result_dir / "index.html", links)
+    assert_links(result_dir / "page1/index.html", links)
+    assert_links(result_dir / "page2/index.html", links)
+    assert_links(result_dir / "subdir/page3/index.html", links)
+
+    assert check_link_targets(result_dir) == len(links) * 4
+    check_reference_targets_exist(result_dir)
+
+
+def test_html_subdir_reference(tmp_path: Path):
+    index = """
+Test project
+============
+
+.. toctree::
+
+   subdir/page1
+   subdir/subdir2/page2
+
+Index Page
+----------
+
+.. code:: python
+
+   import test_project
+   test_project.bar()
+
+.. automodule:: test_project
+"""
+
+    page = """
+Page {idx}
+===========
+
+.. code:: python
+
+   import test_project
+   test_project.bar()
+
+.. autolink-examples:: test_project.bar
+"""
+
+    files = {
+        "conf.py": default_conf,
+        "index.rst": index,
+        "subdir/page1.rst": page.format(idx=1),
+        "subdir/subdir2/page2.rst": page.format(idx=2),
+    }
+    links = ["test_project", "test_project.bar"]
+
+    result_dir = _sphinx_build(tmp_path, "html", files)
+
+    assert_links(result_dir / "index.html", links)
+    assert_links(result_dir / "subdir/page1.html", links)
+    assert_links(result_dir / "subdir/subdir2/page2.html", links)
+
+    assert check_link_targets(result_dir) == len(links) * 3
+    check_reference_targets_exist(result_dir)
+
+
 def _sphinx_build(
     folder: Path, builder: str, files: dict[str, str], n_processes: int | None = None
 ) -> Path:
@@ -286,7 +426,9 @@ def _sphinx_build(
     src_dir = folder / "src"
     src_dir.mkdir(exist_ok=True)
     for name, content in files.items():
-        (src_dir / name).write_text(content, "utf-8")
+        path = src_dir / name
+        path.parent.mkdir(exist_ok=True, parents=True)
+        path.write_text(content, "utf-8")
 
     build_dir = folder / "build"
     args = ["-M", builder, str(src_dir), str(build_dir), "-W"]
